 ###############################################
#                                               #
#    B A S S ' S   S E E N   E X T E N D E D    #
#     v1.0 (21/02/2010)    2010 MenzAgitat     #
#                                               #
#        IRC:  irc.epiknet.org  #boulets        #
#          http://www.boulets-roxx.com          #
#                                               #
#     Mes scripts sont tlchargeables sur      #
#            http://www.eggdrop.fr              #
#        ou  http://www.egghelp.org             #
#                                               #
 ###############################################

#
# Description
#
# Bass's Seen Extended est une extension prvue pour fonctionner conjointement
# avec Bass's Seen v1.4.2c dont elle rutilise la base de donnes, certaines
# procdures et certaines options communes.
# Cette extension a pour but de combler une lacune de Bass's Seen en permettant
# d'effectuer des recherches dans les nicks ou les hosts, et de retourner les
# nicks ou hosts rsultants.
# Pas trs convaincant expliqu comme a, mais essayez-le donc et vous en
# comprendrez trs vite l'utilit.
# Une commande publique et une commande dcc sont disponibles.
#
# Syntaxe (exemple donn avec la commande publique et les options par dfaut) :
#		!whowas <-n/-h> <argument_de_recherche> <nicks/hosts>
#				-n										= demande  effectuer une recherche dans les nicks
#				-h										= demande  effectuer une recherche dans les hosts
#				argument_de_recherche	= ce que vous voulez chercher (jokers accepts)
#				nicks									= affiche une liste des nicks rsultants de la recherche
#				hosts									= affiche une liste des hosts rsultants de la recherche
#

#
# Changelog
#
# 1.0 - 1re version
#

#
# LICENCE:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#

# Note pour les programmeurs :
# Dans la version 1.6.19 d'eggdrop, le numro de version affich par .vbottree
# et [numversion] est incorrect; il affiche 1061800 ou 1061801, ce qui
# correspond  la version 1.6.18. On utilise donc une autre technique pour
# vrifier le numro de version.
if {[info commands BSeenExtended::uninstall] eq "::BSeenExtended::uninstall"} { BSeenExtended::uninstall }
if { [join [split [string range [lindex $version 0] 0 5] "."] ""] < 1619 } { putloglev o * "\00304\002\[BASS'S SEEN EXTENDED - ERREUR\]\002\003 La version de votre eggdrop est \00304\002$version\002\003; \002Bass's Seen Extended\002 ne fonctionnera correctement que sur les eggdrops version 1.6.19 ou suprieure." ; return }
if { [info tclversion] < 8.5 } { putloglev o * "\00304\002\[BASS'S SEEN EXTENDED - ERREUR\]\002\003 \002Bass's Seen Extended\002 ncessite que Tcl 8.5 (ou plus) soit install pour fonctionner. Votre version actuelle de Tcl est \00304\002$tcl_version\002\003." ; return }
package require Tcl 8.5
if { ![info exists ::bs(version)] } { putloglev o * "\00304\002\[BASS'S SEEN EXTENDED - ERREUR\]\002\003 \002Bass's Seen Extended\002 est une extension de \002Bass's Seen\002 et il ne fonctionnera pas si vous n'installez pas ce dernier." ; return }
if { $::bs(version) ne "bseen1.4.2c" } { putloglev o * "\00304\002\[BASS'S SEEN EXTENDED - ERREUR\]\002\003 \002Bass's Seen Extended\002 est conu pour fonctionner avec la version \00304\0021.4.2c\002\003 de Bass's Seen. La version que vous utilisez actuellement est \00304\002$::bs(version)\002\00304." ; return }
namespace eval BSeenExtended {


	###########################
	#				PARAMETRES        #
	###########################

	# Autorisations ncessaires pour pouvoir utiliser les commandes de ce script
	variable whowas_auth "-|-"

	# Commande pour effectuer une recherche dans la base de donnes
	# Remarque : le prfixe de la commande publique est rcupr du paramtre
	# $bs(cmdchar) dfini dans bseen1.4.2.tcl
	variable whowas_cmd "whowas"
	
	# sparateur de rsultats
	variable separator " \00307|\00314 "

	# nombre maximum de rsultats affichs si les rsultats demands sont des nicks
	# (ne concerne pas la version DCC de la commande)
	variable max_nicks_results 15

	# nombre maximum de rsultats affichs si les rsultats demands sont des hosts
	# (ne concerne pas la version DCC de la commande)
	variable max_hosts_results 8

	# Nombre de caractres maximum affichables par ligne. La limite peut varier
	# d'un serveur IRC  un autre. Si la fin de certaines lignes est tronque,
	# cela signifie que vous devez diminuer cette valeur.
	variable max_line_length 440





####################################################################
#                                                                  #
# NE MODIFIEZ RIEN APRES CE CADRE SI VOUS NE CONNAISSEZ PAS LE TCL #
#                                                                  #
#   DO NOT MODIFY ANYTHING BELOW THIS BOX IF YOU DON'T KNOW TCL    #
#                                                                  #
####################################################################

	###
	### Initialisation
	###
  variable scriptname "Bass's Seen Extended"
  variable version "1.0.20100221"
	# Procdure de dsinstallation
	# (le script se dsinstalle totalement avant chaque rehash ou  chaque relecture au moyen de la commande "source" ou autre)
	proc uninstall {args} {
		putlog "Dsallocation des ressources de \002$BSeenExtended::scriptname\002..."
		foreach binding [lsearch -inline -all -regexp [binds *[set ns [string range [namespace current] 2 end]]*] " \{?(::)?$ns"] {
			unbind [lindex $binding 0] [lindex $binding 1] [lindex $binding 2] [lindex $binding 4]
		}
		foreach running_timer [timers] {
			if { [string match "*[namespace current]::*" [lindex $running_timer 1]] } { killtimer [lindex $running_timer 2] }
		}
		foreach running_utimer [utimers] {
			if { [string match "*[namespace current]::*" [lindex $running_utimer 1]] } { killutimer [lindex $running_utimer 2] }
		}
		namespace delete ::BSeenExtended
	}
}

###
### procdure principale
###
proc BSeenExtended::pub_whowas {nick host hand chan arg} {
	if { ([lsearch -nocase $::bs(no_pub) $chan] != -1) || ([::bs_flood [set nick [hand2nick $hand]] [getchanhost $nick]]) } { return }
	if { [lsearch -nocase $::bs(quiet_chan) $chan] == -1 } { set output_mode "PRIVMSG $chan" } { set output_mode "NOTICE $nick" }
	set arg [::tcl::string::tolower $arg]
	if { ([llength $arg] != 3) || (([lindex $arg 0] ne "-n") && ([lindex $arg 0] ne "-h")) || (([lindex $arg 2] ne "nicks") && ([lindex $arg 2] ne "hosts")) } {
		puthelp "$output_mode :\037Syntaxe\037 : \002$::bs(cmdchar)$BSeenExtended::whowas_cmd\002 \00314<\003-n\00314/\003-h\00314> <\003argument_de_recherche\00314> <\003nicks\00314/\003hosts\00314>\003 \00307|\003 \002-n\002 effectue une recherche dans les nicks \00307|\003 \002-h\002 effectue une recherche dans les hosts \00307|\003 \002nicks\002 affiche les nicks rsultants de la recherche \00307|\003 \002hosts\002 affiche les hosts rsultants de la recherche \00304||\003 \037Exemple\037 :\00314 $::bs(cmdchar)$BSeenExtended::whowas_cmd -h *@*.fbx.proxad.net nicks\003"
		return
	}
	BSeenExtended::common_whowas 1 pub $nick $chan $arg
	return
}
proc BSeenExtended::msg_whowas {nick host hand arg} {
	if {[::bs_flood [set nick [hand2nick $hand]] [getchanhost $nick]] } {return}
	set arg [::tcl::string::tolower $arg]
	if { [llength $arg] != 3 || (([lindex $arg 0] ne "-n") && ([lindex $arg 0] ne "-h")) || (([lindex $arg 2] ne "nicks") && ([lindex $arg 2] ne "hosts")) } {
		puthelp "PRIVMSG $nick :\037Syntaxe\037 : \002$BSeenExtended::whowas_cmd\002 \00314<\003-n\00314/\003-h\00314> <\003argument_de_recherche\00314> <\003nicks\00314/\003hosts\00314>\003 \00307|\003 \002-n\002 effectue une recherche dans les nicks \00307|\003 \002-h\002 effectue une recherche dans les hosts \00307|\003 \002nicks\002 affiche les nicks rsultants de la recherche \00307|\003 \002hosts\002 affiche les hosts rsultants de la recherche \00304|\003 \037Exemple\037 :\00314 $BSeenExtended::whowas_cmd -h *@*.fbx.proxad.net nicks\003"
		return
	}
	BSeenExtended::common_whowas 1 msg $nick - $arg
	return
}
proc BSeenExtended::dcc_whowas {hand idx arg} {
	if {[::bs_flood [set nick [hand2nick $hand]] [getchanhost $nick]] } {return}
	set arg [::tcl::string::tolower $arg]
	if { [llength $arg] != 3 || (([lindex $arg 0] ne "-n") && ([lindex $arg 0] ne "-h")) || (([lindex $arg 2] ne "nicks") && ([lindex $arg 2] ne "hosts")) } {
		putdcc $idx "\037Syntaxe\037 : \002.$BSeenExtended::whowas_cmd\002 \00314<\003-n\00314/\003-h\00314> <\003argument_de_recherche\00314> <\003nicks\00314/\003hosts\00314>\003 \00307|\003 \002-n\002 effectue une recherche dans les nicks \00307|\003 \002-h\002 effectue une recherche dans les hosts \00307|\003 \002nicks\002 affiche les nicks rsultants de la recherche \00307|\003 \002hosts\002 affiche les hosts rsultants de la recherche \00304|\003 \037Exemple\037 :\00314 .$BSeenExtended::whowas_cmd -h *@*.fbx.proxad.net nicks\003"
		return
	}
	BSeenExtended::common_whowas 0 dcc $hand $idx $arg
	return
}
proc BSeenExtended::common_whowas {limit_results coming_from nick chan arg} {
	lassign $arg search_mode search_argument results_mode
	set search_argument [regsub -all {["\]\[\}\{\\]} $search_argument "\\\\&"]            ;#"
	set array_search_id [array startsearch ::bs_list]
	if { $search_mode eq "-n" } { set index 0 } { set index 1 }
	while { [array anymore ::bs_list $array_search_id] } {
		set db_index [array nextelement ::bs_list $array_search_id]
		if { ([::tcl::string::match -nocase $search_argument [lindex $::bs_list($db_index) $index]]) || ((!$index) && ([lindex $::bs_list($db_index) 3] eq "nick") && ([::tcl::string::match -nocase $search_argument [lindex $::bs_list($db_index) 5]])) } {
			lappend intermediate_results $::bs_list($db_index)
		}
	}
	array donesearch ::bs_list $array_search_id
	if { [info exists intermediate_results] } {
		# on classe les rsultats par ordre chronologique inverse
		set intermediate_results [lsort -decreasing -index 2 $intermediate_results]
		set results ""
		if { $results_mode eq "nicks" } {
			set index 0
			set max_results $BSeenExtended::max_nicks_results
		} else {
			set index 1
			set max_results $BSeenExtended::max_hosts_results
		}
		# on rcupre uniquement les donnes qui nous intressent (hosts ou nicks)
		# dans les rsultats intermdiaires et on limine les doublons au passage
		foreach result_element $intermediate_results {
			if { [lsearch -exact $results [set result [lindex $result_element $index]]] == -1 } {
				lappend results $result
			}
			if { (!$index) && ([lindex $result_element 3] eq "nick") && ([lsearch -exact $results [set result [lindex $result_element 5]]] == -1) } {
				lappend results $result
			}
		}
		set total_results [llength $results]
		if { $limit_results } {
			if { $total_results > $max_results } { set displayed_results $max_results } { set displayed_results $total_results }
			if { $displayed_results == 1 } {
				if { $total_results == 1 } {
					set output "\003\0021\002 rsultat :\00314 [join $results]\003"
				} else {
					set output "\003\002$total_results\002 rsultats. Voici le plus rcent :\00314 [join [lindex $results 0]]\003"
				}
			} else { 				
				if { $total_results > $displayed_results } {
					set output "\003\002$total_results\002 rsultats. Voici les $displayed_results plus rcents (dans l'ordre) :\00314 [join [lrange $results 0 [expr $displayed_results -1]] $BSeenExtended::separator]\003"
				} else {
					set output "\003\002$total_results\002 rsultats (du plus rcent au plus ancien) :\00314 [join $results $BSeenExtended::separator]\003"
				}
			}
		} else {
			if { $total_results == 1 } {
				set output "\003\002$total_results\002 rsultat :\00314 [join $results]\003"
			} else {
				set output "\003\002$total_results\002 rsultats (du plus rcent au plus ancien) :\00314 [join $results $BSeenExtended::separator]\003"
			}
		}
	} else {
		set output "$nick > La recherche n'a rien donn."
	}
	BSeenExtended::output_split_line $BSeenExtended::max_line_length $coming_from $nick $chan $output
	return
}

###
### Dcoupage et affichage d'une ligne trop longue en plusieurs lignes
###
proc BSeenExtended::output_split_line {limit coming_from nick chan text} {
	set output_length [string length $text]
	set letter_index 0
	while {$letter_index < $output_length} {
		if {$output_length - $letter_index > $limit} {
			set cut_index [string last " " $text [expr $letter_index + $limit ]]
 		} else {
			set cut_index $output_length
		}
		lappend output [concat \00314 [string range $text $letter_index $cut_index]]
		set letter_index $cut_index
	}
	foreach line $output {
		if { $coming_from eq "pub" } {
			if { [lsearch -nocase $::bs(quiet_chan) $chan] == -1 } {
				puthelp "PRIVMSG $chan :$line"
			} else {
				puthelp "NOTICE $nick :$line"
			}
		} elseif { $coming_from eq "msg" } {
			puthelp "PRIVMSG $nick :$line"
		} else {						
			putdcc $chan "$line"
		}
	}
	return
}

###
### binds
###
bind pub $BSeenExtended::whowas_auth $::bs(cmdchar)$BSeenExtended::whowas_cmd BSeenExtended::pub_whowas
bind msg $BSeenExtended::whowas_auth $BSeenExtended::whowas_cmd BSeenExtended::msg_whowas
bind dcc $BSeenExtended::whowas_auth $BSeenExtended::whowas_cmd BSeenExtended::dcc_whowas
bind evnt - prerehash BSeenExtended::uninstall


putlog "\002$BSeenExtended::scriptname v$BSeenExtended::version\002 (2010 MenzAgitat) a t charg."
